@@ -6,7 +6,10 @@ from sqlalchemy import create_engine
 
 from flytekit import current_context, kwtypes
 from flytekit.core.base_sql_task import SQLTask
-from flytekit.core.python_function_task import PythonInstanceTask
+from flytekit.core.context_manager import SerializationSettings
+from flytekit.core.python_customized_container_task import PythonCustomizedContainerTask
+from flytekit.core.shim_task import ShimTaskExecutor
+from flytekit.models import task as task_models
 from flytekit.models.security import Secret
 from flytekit.types.schema import FlyteSchema
 
@@ -33,7 +36,7 @@ class SQLAlchemyConfig(object):
     secret_connect_args: typing.Optional[typing.Dict[str, Secret]] = None
 
 
-class SQLAlchemyTask(PythonInstanceTask[SQLAlchemyConfig], SQLTask[SQLAlchemyConfig]):
+class SQLAlchemyTask(PythonCustomizedContainerTask[SQLAlchemyConfig], SQLTask[SQLAlchemyConfig]):
     """
     Makes it possible to run client side SQLAlchemy queries that optionally return a FlyteSchema object
 
@@ -54,13 +57,12 @@ class SQLAlchemyTask(PythonInstanceTask[SQLAlchemyConfig], SQLTask[SQLAlchemyCon
     ):
         output_schema = output_schema_type if output_schema_type else FlyteSchema
         outputs = kwtypes(results=output_schema)
-        self._uri = task_config.uri
-        self._connect_args = task_config.connect_args or {}
-        self._secret_connect_args = task_config.secret_connect_args
 
         super().__init__(
             name=name,
             task_config=task_config,
+            container_image="ghcr.io/flyteorg/flytekit:sqlalchemy-6deb81af74ce8f3768553c188ab35660c717420a",
+            executor_type=SQLAlchemyTaskExecutor,
             task_type=self._SQLALCHEMY_TASK_TYPE,
             query_template=query_template,
             inputs=inputs,
@@ -73,13 +75,27 @@ class SQLAlchemyTask(PythonInstanceTask[SQLAlchemyConfig], SQLTask[SQLAlchemyCon
         c = self.python_interface.outputs["results"].column_names()
         return c if c else None
 
-    def execute(self, **kwargs) -> typing.Any:
-        if self._secret_connect_args is not None:
-            for key, secret in self._secret_connect_args.items():
+    def get_custom(self, settings: SerializationSettings) -> typing.Dict[str, typing.Any]:
+        return {
+            "query_template": self.query_template,
+            "uri": self.task_config.uri,
+            "connect_args": self.task_config.connect_args or {},
+            "secret_connect_args": self.task_config.secret_connect_args,
+        }
+
+
+class SQLAlchemyTaskExecutor(ShimTaskExecutor[SQLAlchemyTask]):
+    def execute_from_model(self, tt: task_models.TaskTemplate, **kwargs) -> typing.Any:
+        if tt.custom["secret_connect_args"] is not None:
+            for key, secret in tt.custom["secret_connect_args"].items():
                 value = current_context().secrets.get(secret.group, secret.key)
-                self._connect_args[key] = value
-        engine = create_engine(self._uri, connect_args=self._connect_args, echo=False)
-        print(f"Connecting to db {self._uri}")
+                tt.custom["connect_args"][key] = value
+
+        engine = create_engine(tt.custom["uri"], connect_args=tt.custom["connect_args"], echo=False)
+        print(f"Connecting to db {tt.custom['uri']}")
+
+        interpolated_query = SQLAlchemyTask.interpolate_query(tt.custom["query_template"], **kwargs)
+        print(f"Interpolated query {interpolated_query}")
         with engine.begin() as connection:
-            df = pd.read_sql_query(self.get_query(**kwargs), connection)
+            df = pd.read_sql_query(interpolated_query, connection)
         return df
